import {
  Body,
  Controller,
  ForbiddenException,
  Get,
  Post,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Operador } from '../operador/entity/operador.entity';
import { Serealize } from '../interceptors/serialize.interceptor';
import { CarritoService } from './carrito.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { CarritoDto } from './dto/input/carrito.dto';
import { CarritosDto } from './dto/input/carritos.dto';
import { CreateCarritoDto } from './dto/input/create.dto';
import { UpdateCarritoDto } from './dto/input/update.dto';
import { MessageOutputDto } from '../dto/output/message.dto';
import { CarritoOutputDto } from './dto/output/carrito.dto';
import { CarritosOutputDto } from './dto/output/carritos.dto';

@Controller('carrito')
@ApiTags('carrito')
export class CarritoController {
  constructor(
    private carritoService: CarritoService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(CarritoOutputDto)
  @Get('carrito')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Información de un carrito.' })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id del carrito.',
    name: 'id_carrito',
    type: 'string',
  })
  carrito(@Request() req, @Query() query: CarritoDto) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(operador);
    return this.carritoService.findFullInfoById(
      operador,
      parseInt(query.id_carrito),
    );
  }

  @Serealize(CarritosOutputDto)
  @Get('carritos')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Información de 25 carritos dependiendo de los filtros enviados.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Carritos activos. Boolean string.',
    name: 'activo',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Carrito que se desea buscar.',
    name: 'carrito',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id de la marca.',
    name: 'id_marca',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del modelo.',
    name: 'id_modelo',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del módulo.',
    name: 'id_modulo',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del tipo de carrito.',
    name: 'id_tipo_carrito',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Página en la que se encuentra el operador.',
    name: 'pagina',
    type: 'string',
    required: false,
  })
  carritos(@Request() req, @Query() query: CarritosDto) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(operador);
    return this.carritoService.findFullInfoAll(operador, query);
  }

  @Serealize(MessageOutputDto)
  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Crear un carrito.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: {
      ejemplo: {
        value: {
          carrito: 'C',
          id_tipo_carrito: 1,
          id_marca: 1,
          id_modelo: 1,
          id_modulo: 1,
        },
      },
    },
  })
  create(@Request() req, @Body() body: CreateCarritoDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.carritoService.create(
      admin,
      body.carrito,
      body.id_modulo,
      body.id_tipo_carrito,
      body.id_marca,
      body.id_modelo,
    );
  }

  @Serealize(MessageOutputDto)
  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Actualizar información de un carrito.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description:
      'Variables que necesita el endpoint. Las variables con "_" al inicio son opcionales.',
    examples: {
      ejemplo: {
        value: {
          id_carrito: 1,
          _activo: true,
          _carrito: '',
          _laboratorioMovil: true,
          _motivo: '',
        },
      },
    },
  })
  update(@Request() req, @Body() body: UpdateCarritoDto) {
    const data = { ...body };
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(operador);
    delete data.motivo;
    delete data.laboratorioMovil;
    return this.carritoService.update(
      operador,
      data,
      body.motivo,
      body.laboratorioMovil,
    );
  }
}
