import {
  ConflictException,
  ForbiddenException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { InstitucionTipoEntrada } from './entity/institucion-tipo-entrada.entity';
import { Operador } from '../operador/entity/operador.entity';
import { TipoEntrada } from './entity/tipo-entrada.entity';
import { Usuario } from 'src/usuario/entity/usuario.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionUsuarioService } from '../institucion-usuario/institucion-usuario.service';

@Injectable()
export class InstitucionTipoEntradaService {
  constructor(
    @InjectRepository(InstitucionTipoEntrada)
    private institucionTipoEntradaRepository: Repository<InstitucionTipoEntrada>,
    @InjectRepository(TipoEntrada)
    private tipoEntradaRepository: Repository<TipoEntrada>,
    private institucionService: InstitucionService,
    private institucionUsuarioService: InstitucionUsuarioService,
  ) {}

  create(tipo_entrada: string): Promise<{ message: string }> {
    // Buscamos un registro con ese nombre
    return this.tipoEntradaRepository
      .findOne({ where: { tipo_entrada } })
      .then((existeTipoEntrada) => {
        // Sacamos error si existe
        if (existeTipoEntrada)
          throw new ConflictException('Ya existe este tipo de conector.');
        // Creamos y guardamos un registro
        return this.tipoEntradaRepository.save(
          this.tipoEntradaRepository.create({ tipo_entrada }),
        );
      })
      .then(async (tipoEntrada) => {
        const instituciones = await this.institucionService.findAll();

        // Asociamos el nuevo tipo de entrada con las instituciones existentes
        for (let i = 0; i < instituciones.length; i++)
          await this.institucionTipoEntradaRepository.save(
            this.institucionTipoEntradaRepository.create({
              institucion: instituciones[i],
              tipoEntrada,
            }),
          );
        return { message: 'Se creó correctamente un nuevo tipo de conector.' };
      });
  }

  findAll(): Promise<TipoEntrada[]> {
    return this.tipoEntradaRepository.find({ order: { tipo_entrada: 'ASC' } });
  }

  async findAllMostrar(
    usuario: Usuario,
    id_institucion: number,
  ): Promise<InstitucionTipoEntrada[]> {
    const institucion = await this.institucionService.findById(id_institucion);
    const institucionUsuario =
      await this.institucionUsuarioService.findByIdUsuarioIdInstitucion(
        usuario,
        institucion,
      );

    if (!institucionUsuario)
      throw new ForbiddenException(
        'No puedes acceder a esta información porque no peteneces a esta institución.',
      );
    return this.institucionTipoEntradaRepository.find({
      where: { mostrar: true, institucion },
      order: { tipoEntrada: { tipo_entrada: 'ASC' } },
    });
  }

  findAllOperador(operador: Operador): Promise<InstitucionTipoEntrada[]> {
    return this.institucionTipoEntradaRepository.find({
      where: { institucion: operador.institucion },
      order: { tipoEntrada: { tipo_entrada: 'ASC' } },
    });
  }

  findById(
    id_institucion_tipo_entrada: number,
  ): Promise<InstitucionTipoEntrada> {
    return this.institucionTipoEntradaRepository
      .findOne({ where: { id_institucion_tipo_entrada } })
      .then((institucionTipoEntrada) => {
        if (!institucionTipoEntrada)
          throw new NotFoundException(
            'No existe este id institución tipo entrada.',
          );
        return institucionTipoEntrada;
      });
  }

  findTipoEntradaById(id_tipo_entrada: number): Promise<TipoEntrada> {
    return this.tipoEntradaRepository
      .findOne({ where: { id_tipo_entrada } })
      .then((tipoEntrada) => {
        if (!tipoEntrada)
          throw new NotFoundException('No existe este id tipo de entrada.');
        return tipoEntrada;
      });
  }

  findTipoEntradaByTipoEntrada(tipo_entrada: string): Promise<TipoEntrada> {
    return this.tipoEntradaRepository.findOne({ where: { tipo_entrada } });
  }

  update(
    admin: Operador,
    attrs: Partial<InstitucionTipoEntrada>,
  ): Promise<{ message: string }> {
    return this.findById(attrs.id_institucion_tipo_entrada)
      .then((institucionTipoEntrada) => {
        // Validamos que la institucion tipo entrada pertenezca a la institución del admin
        if (
          admin.institucion.id_institucion !=
          institucionTipoEntrada.institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes modificar la información este tipo de conector porque no pertenece a tu institución.',
          );
        // Asignamos valores enviados al objeto
        Object.assign(institucionTipoEntrada, attrs);
        // Guardamos cambios
        return this.institucionTipoEntradaRepository.save(
          institucionTipoEntrada,
        );
      })
      .then((_) => ({ message: 'Se guardaron los cambios correctamente.' }));
  }
}
